from pip._internal.req import parse_requirements
from setuptools import setup, find_packages

VERSION = '0.0.7'
DESCRIPTION = 'Zeblok Python SDK'
LONG_DESCRIPTION = 'Zeblok Python SDK to interact the Zeblok Ai-MicroCloud'

setup(
    name="zeblok-py",
    version=VERSION,
    author="Karan Pathak",
    author_email="karan@dataturtles.com",
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    url="",
    packages=find_packages(),
    install_requires=[item.requirement for item in parse_requirements('requirements.txt', session=False)],
    python_requires=">=3.9",
    keywords=['python', 'zeblok'],
    classifiers=[
        "Programming Language :: Python :: 3",
    ],
)